@@ -23,11 +23,28 @@
 #
 # author: Steven Czerwinski <czerwin@scalyr.com>
 
-__author__ = 'czerwin@scalyr.com'
+from __future__ import unicode_literals
+from __future__ import absolute_import
+
+__author__ = "czerwin@scalyr.com"
+
+if False:
+    # NOTE: This is a workaround for old Python versions where typing module is not available
+    # We should eventually improve that once we start producing distributions with Python
+    # interpreter and dependencies bundled in.
+    # Adding conditional "typing" dependency would require too much boiler plate code at this point.
+    from typing import List
+    from typing import Any
+
+import six
+from six.moves import range
+
 
 from scalyr_agent.json_lib.exceptions import JsonConversionException
 from scalyr_agent.json_lib.exceptions import JsonMissingFieldException
 
+# 2->TODO remove 'str' where is the type check
+
 
 class JsonObject(object):
     """Represents a JSON object, mapping keys to values.
@@ -52,17 +69,21 @@ class JsonObject(object):
             self.__map = {}
         else:
             self.__map = content
-        for key, value in key_values.iteritems():
-            self.__map[key] = value
+        # 2->TODO convert keys to unicode.
+        for key, value in six.iteritems(key_values):
+            self.__map[six.ensure_text(key)] = value
 
     def to_json(self):
         """Returns a string containing the JSON representation of this object.
-         
+
         Returns the string representation.  If there were comments or other
         non-standard JSON elements in the input that created this object, they
         will not be included."""
         # ???? TODO
 
+    def __repr__(self):
+        return repr(self.__map)
+
     def __len__(self):
         """Returns the number of keys in the JsonObject"""
         return len(self.__map)
@@ -98,35 +119,41 @@ class JsonObject(object):
         return self
 
     def __iter__(self):
-        return self.__map.iterkeys()
+        return six.iterkeys(self.__map)
+
+    def update(self, other):
+        """Updates the map with key/value pairs from other.  Overwriting existing keys"""
+        return self.__map.update(other)
 
     def iteritems(self):
         """Returns an iterator over the items (key/value tuple) for this object."""
-        return self.__map.iteritems()
+        return six.iteritems(self.__map)
 
     def itervalues(self):
         """Returns an iterator over the values for this object."""
-        return self.__map.itervalues()
+        return six.itervalues(self.__map)
 
     def iterkeys(self):
         """Returns an iterator over the keys for this object."""
-        return self.__map.iterkeys()
+        return six.iterkeys(self.__map)
 
     def items(self):
         """Returns a list of items (key/value tuple) for this object."""
-        return self.__map.items()
+        return list(self.__map.items())
 
     def values(self):
         """Returns a list of values for this object."""
-        return self.__map.values()
+        return list(self.__map.values())
 
     def keys(self):
         """Returns a list keys for this object."""
-        return self.__map.keys()
+        return list(self.__map.keys())
 
     def __getitem__(self, field):
-        if not field in self:
-            raise JsonMissingFieldException('The missing field "%s" in JsonObject.' % field)
+        if field not in self:
+            raise JsonMissingFieldException(
+                'The missing field "%s" in JsonObject.' % field
+            )
         return self.__map[field]
 
     def copy(self):
@@ -134,6 +161,18 @@ class JsonObject(object):
         result.__map = self.__map.copy()
         return result
 
+    def to_dict(self):
+        """Creates a copy of this object as represented using `dict` and `list` objects instead
+        of `JsonObject` and `JsonArray`.  This will be a deep copy.
+
+        Note: This function may not be performant (it is recursive) so you may wish to not use it
+        on critical code paths.
+
+        :return: The dict version of this JSON object.
+        :rtype: dict
+        """
+        return convert_to_builtin_type(self)
+
     def get(self, field, default_value=None, none_if_missing=False):
         """Returns the specified field without any conversion.
 
@@ -153,9 +192,8 @@ class JsonObject(object):
         @raise JsonMissingFieldException: If the underlying value was not present and no other value was specified to
             be returned using default_value or none_if_missing."""
 
-        if not field in self:
-            return self.__compute_missing_value(
-                field, default_value, none_if_missing)
+        if field not in self:
+            return self.__compute_missing_value(field, default_value, none_if_missing)
 
         return self.__map[field]
 
@@ -185,21 +223,18 @@ class JsonObject(object):
         @raise JsonMissingFieldException: If the underlying value was not present and no other value was specified to
             be returned using default_value or none_if_missing."""
 
-        if not field in self:
-            return self.__compute_missing_value(
-                field, default_value, none_if_missing)
+        if field not in self:
+            return self.__compute_missing_value(field, default_value, none_if_missing)
 
         value = self.__map[field]
         value_type = type(value)
         if value_type is bool:
             return value
-        elif value_type is int:
-            return self.__num_to_bool(field, float(value))
-        elif value_type is long:
+        elif value_type in six.integer_types:
             return self.__num_to_bool(field, float(value))
         elif value_type is float:
             return self.__num_to_bool(field, value)
-        elif value_type is str or value_type is unicode:
+        elif value_type is six.text_type:
             return not value == "" and not value == "f" and not value == "false"
         else:
             return self.__conversion_error(field, value, "boolean")
@@ -214,12 +249,12 @@ class JsonObject(object):
             raises a JsonConversionError.
 
         @raise JsonConversionError: If the value is not either zero or one."""
-        if abs(value) < 1E-10:
+        if abs(value) < 1e-10:
             return False
-    
-        if abs(1 - value) < 1E-10:
+
+        if abs(1 - value) < 1e-10:
             return True
-    
+
         return self.__conversion_error(field, value, "boolean")
 
     def get_int(self, field, default_value=None, none_if_missing=False):
@@ -249,20 +284,22 @@ class JsonObject(object):
         @raise JsonMissingFieldException: If the underlying value was not present and no other value was specified to
             be returned using default_value or none_if_missing."""
 
-        if not field in self:
-            return self.__compute_missing_value(
-                field, default_value, none_if_missing)
+        if field not in self:
+            return self.__compute_missing_value(field, default_value, none_if_missing)
 
         value = self.__map[field]
         value_type = type(value)
 
-        if (value_type is int or value_type is long or value_type is float or value_type is str
-                or value_type is unicode):
+        if (
+            value_type in six.integer_types
+            or value_type is float
+            or value_type is six.text_type
+        ):
             try:
                 # If it is a string type, then try to convert to a float
                 # first, and then int.. that way we will just truncate the
                 # float.
-                if value_type is str or value_type is unicode:
+                if value_type is six.text_type:
                     value = float(value)
                 return int(value)
             except ValueError:
@@ -270,6 +307,7 @@ class JsonObject(object):
         else:
             return self.__conversion_error(field, value, "integer")
 
+    # 2->TODO: Should we keep this?
     def get_long(self, field, default_value=None, none_if_missing=False):
         """Returns the specified field as a long with some conversion.
 
@@ -297,22 +335,24 @@ class JsonObject(object):
         @raise JsonMissingFieldException: If the underlying value was not present and no other value was specified to
             be returned using default_value or none_if_missing."""
 
-        if not field in self:
-            return self.__compute_missing_value(
-                field, default_value, none_if_missing)
-    
+        if field not in self:
+            return self.__compute_missing_value(field, default_value, none_if_missing)
+
         value = self.__map[field]
         value_type = type(value)
 
-        if (value_type is int or value_type is long or value_type is float or value_type is str or
-                value_type is unicode):
+        if (
+            value_type in six.integer_types
+            or value_type is float
+            or value_type is six.text_type
+        ):
             try:
                 # If it is a string type, then try to convert to a float
                 # first, and then long.. that way we will just truncate the
                 # float.
-                if value_type is str or value_type is unicode:
+                if value_type is six.text_type:
                     value = float(value)
-                return long(value)
+                return int(value)
             except ValueError:
                 return self.__conversion_error(field, value, "long")
         else:
@@ -342,15 +382,17 @@ class JsonObject(object):
         @raise JsonMissingFieldException: If the underlying value was not present and no other value was specified to
             be returned using default_value or none_if_missing."""
 
-        if not field in self:
-            return self.__compute_missing_value(
-                field, default_value, none_if_missing)
-    
+        if field not in self:
+            return self.__compute_missing_value(field, default_value, none_if_missing)
+
         value = self.__map[field]
         value_type = type(value)
 
-        if (value_type is int or value_type is long or value_type is float or value_type is str or
-                value_type is unicode):
+        if (
+            value_type in six.integer_types
+            or value_type is float
+            or value_type is six.text_type
+        ):
             try:
                 return float(value)
             except ValueError:
@@ -376,9 +418,8 @@ class JsonObject(object):
 
         @raise JsonMissingFieldException: If the underlying value was not present and no other value was specified to
             be returned using default_value or none_if_missing."""
-        if not field in self:
-            return self.__compute_missing_value(
-                field, default_value, none_if_missing)
+        if field not in self:
+            return self.__compute_missing_value(field, default_value, none_if_missing)
 
         value = self.__map[field]
         if value is None:
@@ -386,9 +427,12 @@ class JsonObject(object):
 
         value_type = type(value)
 
-        if (value_type is int or value_type is long or value_type is float or value_type is str or
-                value_type is unicode):
-            return str(value)
+        if (
+            value_type in six.integer_types
+            or value_type is float
+            or value_type is six.text_type
+        ):
+            return six.text_type(value)
         else:
             return self.__conversion_error(field, value, "str")
 
@@ -411,9 +455,8 @@ class JsonObject(object):
         @raise JsonConversionError: If the underlying field's value is not a JsonObject.
         @raise JsonMissingFieldException: If the underlying value was not present and no other value was specified to
             be returned using default_value or none_if_missing."""
-        if not field in self:
-            return self.__compute_missing_value(
-                field, default_value, none_if_missing)
+        if field not in self:
+            return self.__compute_missing_value(field, default_value, none_if_missing)
         value = self.__map[field]
         if isinstance(value, JsonObject):
             return value
@@ -430,7 +473,7 @@ class JsonObject(object):
             object, and returns it.
 
         @raise JsonConversionError: If the underlying field's value is not a JsonObject."""
-        if not field in self:
+        if field not in self:
             self.__map[field] = JsonObject()
         value = self.__map[field]
         if isinstance(value, JsonObject):
@@ -457,9 +500,8 @@ class JsonObject(object):
         @raise JsonConversionError: If the underlying field's value is not a JsonArray.
         @raise JsonMissingFieldException: If the underlying value was not present and no other value was specified to
             be returned using default_value or none_if_missing."""
-        if not field in self:
-            return self.__compute_missing_value(
-                field, default_value, none_if_missing)
+        if field not in self:
+            return self.__compute_missing_value(field, default_value, none_if_missing)
         value = self.__map[field]
         if isinstance(value, JsonArray):
             return value
@@ -480,10 +522,10 @@ class JsonObject(object):
         @raise JsonConversionException: The conversion error."""
         raise JsonConversionException(
             "Failed converting %s of type %s for field %s to desired type %s"
-            % (str(value), type(value), field_name, desired_type))
+            % (six.text_type(value), type(value), field_name, desired_type)
+        )
 
-    def __compute_missing_value(self, field_name, default_value,
-                                none_if_missing):
+    def __compute_missing_value(self, field_name, default_value, none_if_missing):
         """Perform the appropriate action for a missing value.
 
         @param field_name: The name of the missing field.
@@ -501,7 +543,8 @@ class JsonObject(object):
         if none_if_missing:
             return None
         raise JsonMissingFieldException(
-            "The required field %s was not found in object." % field_name)
+            "The required field %s was not found in object." % field_name
+        )
 
     def __eq__(self, other):
         if other is None:
@@ -525,21 +568,27 @@ class JsonArray(object):
 
     def __init__(self, *args):
         """Inits a JsonArray.
-        
+
+        @param content: A dict containing the key/values pairs to use.
         @param *args: The elements to insert into the list."""
-        self.__items = []
+
+        self._items = []
+
         for arg in args:
-            self.__items.append(arg)
+            self._items.append(arg)
+
+    def __repr__(self):
+        return repr(self._items)
 
     def __len__(self):
         """Returns the number of elements in the JsonArray"""
-        return len(self.__items)
+        return len(self._items)
 
     def get_json_object(self, index):
         """Returns the value at the specified index as a JsonObject.
 
         @param index: The index to lookup
-        
+
         @return: The JsonObject at the specified index.
 
         @raise JsonConversionException: If the entry is not a JsonObject
@@ -553,14 +602,16 @@ class JsonArray(object):
         """Returns the value at the specified index.
 
         @param index: The index to lookup
-        
+
         @return: The value at the specified index.
 
         @raise IndexError: If there is no entry at that index."""
-        if index >= len(self.__items) or index < 0:
+        if index >= len(self._items) or index < 0:
             raise IndexError(
-                "The index=%i is out of bounds of array size=%i" % (index, len(self.__items)))
-        return self.__items[index]
+                "The index=%i is out of bounds of array size=%i"
+                % (index, len(self._items))
+            )
+        return self._items[index]
 
     def __setitem__(self, index, value):
         """Sets the value at the specified index.
@@ -569,29 +620,31 @@ class JsonArray(object):
         @param value: The value
 
         @raise IndexError: If there is no entry at that index."""
-        if index >= len(self.__items) or index < 0:
+        if index >= len(self._items) or index < 0:
             raise IndexError(
-                "The index=%i is out of bounds of array size=%i" % (index, len(self.__items)))
-        self.__items[index] = value
+                "The index=%i is out of bounds of array size=%i"
+                % (index, len(self._items))
+            )
+        self._items[index] = value
 
     def add(self, value):
         """Inserts a new element at the end of the array.
 
         @param value: The value to append."""
-        self.__items.append(value)
+        self._items.append(value)
 
     def __iter__(self):
         """Yields all items in the array"""
-        for element in self.__items:
+        for element in self._items:
             yield element
 
     def json_objects(self):
-        """Yields all items in the array, 
+        """Yields all items in the array,
         checking to make sure they are JsonObjects.
 
         @raise JsonConversionException: If an item is reached that is not a JsonObject."""
-        for index in range(len(self.__items)):
-            element = self.__items[index]
+        for index in range(len(self._items)):
+            element = self._items[index]
             if not isinstance(element, JsonObject):
                 yield self.__raise_not_json_object(index)
             else:
@@ -602,9 +655,51 @@ class JsonArray(object):
             return False
         if type(self) is not type(other):
             return False
-        assert isinstance(other.__items, list)
-        return self.__items == other.__items
+        assert isinstance(other._items, list)
+        return self._items == other._items
 
     def __raise_not_json_object(self, index):
         raise JsonConversionException(
-            "A non-JsonObject entry was found in array at index=%i" % index)
\ No newline at end of file
+            "A non-JsonObject entry was found in array at index=%i" % index
+        )
+
+
+class ArrayOfStrings(JsonArray):
+
+    separators = [","]
+
+    def __init__(self, values=None):
+        """Imposes additional constraint that each element must be a string
+        @raise TypeError if an element is not a string
+        """
+        self._items = []
+        if values:
+            for val in values:
+                if type(val) is not six.text_type:
+                    raise TypeError(
+                        "A non-string element was found: %s" % six.text_type(val)
+                    )
+                self._items.append(val)
+
+
+class SpaceAndCommaSeparatedArrayOfStrings(ArrayOfStrings):
+    separators = [None, ","]  # type: List[Any]
+
+
+def convert_to_builtin_type(value):
+    """
+    Convert the provided JSON value for the native / builtin Python type.
+    """
+    value_type = type(value)
+    if value_type == JsonObject or value_type == dict:
+        result = dict()
+        for key, value in six.iteritems(value):
+            result[key] = convert_to_builtin_type(value)
+        return result
+    elif value_type == JsonArray or value_type == list:
+        result = []
+        for x in value:
+            result.append(convert_to_builtin_type(x))
+        return result
+    else:
+        return value
